/*
  How to execute the program?
  1. Compile and create executable
     gcc -Wall inputgen.c -o inputgen
  2. Execute the program with a number
     ./inputgen 20 => produce twenty activities
  3. To save the output to a file
     ./inputgen 20 > datafile
 */

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
const int firstYear = 2009; 
const int lastYear = 2011; /* no leap year */
const char * friendNames[] = 
  {
    "AmyJones",     
    "BillDavis", 
    "CathyBaker",
    "DavidLee", 
    "EmilyCollins", 
    "EthanGray",
    "IsabellaMiller",
    "JacobMartin", 
    "KarenWilliams",
    "LisaCook", 
    "MaryJohnson", 
    "NancyHarris",
    "PaulGreen",
    "RichardBrown",
    "RobertMorgon",
    "SharonThompson"
  };

const char * streetNames [] =  
  {
    "ArmstrongAvenue",
    "BakerStreet",
    "CanalStreet",
    "DowAvenue",
    "FirstStreet",
    "GoldenCourt",
    "HarborAvenue",
    "LexingtonAvenue",
    "MainStreet",
    "NorthwesternAvenue",
    "OakRoad",
    "ParkRoad",
    "StevensonStreet",
    "ThirdStreet",
    "VictorCourt",
    "WashingtonStreet",
  };

const int dateMonth[] = 
  {
    0,
    31,
    28,
    31,
    30,
    31,
    30,
    31,
    31,
    30,
    31,
    30,
    31
  };

const int numFriendNames = sizeof(friendNames) / sizeof(char *);
const int numStreetNames = sizeof(streetNames) / sizeof(char *);
const int numMonths = sizeof(dateMonth) / sizeof(int);

void printActivity(int year, int month, int date)
{
  int activity = rand() % 100;
  if (activity < 8)
    {
      int askyear;
      int askmonth;
      int askdate;
      printf("Q ");
      askyear = firstYear + (rand() % (year - firstYear + 1));
      do {
	askmonth = (rand() % 12) + 1;
      } while ((year == firstYear) && (askmonth > month));
      askdate = (rand() % dateMonth[month]) + 1;
      printf("%d/%d/%d\n", askyear, askmonth, askdate);
      return;
    }
  if (activity < 16)
    {
      int friend1 = rand() % numFriendNames;
      printf("Q ");
      printf("%s\n", friendNames[friend1]); 
      return;
    }
  if (activity < 50)
    {
      int friend1 = rand() % numFriendNames;
      int friend2;
      printf("A ");
      do
	{
	  friend2 = rand() % numFriendNames;
	} while (friend1 == friend2);
      printf("%d/%d/%d ", year, month, date);
      printf("%s ", friendNames[friend1]); 
      printf("meets ");
      printf("%s\n", friendNames[friend2]); 
      return;
    }
  if (activity < 60)
    {
      int friend1 = rand() % numFriendNames;
      printf("R ");
      printf("%s leaves town\n", friendNames[friend1]); 
      return;
    }
  if (activity < 100)
    {
      int friend1 = rand() % numFriendNames;
      printf("A ");
      printf("%d/%d/%d ", year, month, date);
      printf("%s ", friendNames[friend1]); 
      printf("visits ");
      printf("%s\n", streetNames[rand() % numStreetNames]);
      return;
    }
}

int main(int argc, char * argv[])
{
  int numCommand = 0;
  int cmdCnt;
  int month = 1;
  int date = 1;
  int year = firstYear;
  if (argc < 2) 
    {
      printf("number of activities\n");
      return -1;
    }
  numCommand = strtol(argv[1], NULL, 10); 
  if (numCommand <= 0)
    {
      fprintf(stderr, "must be a positive number\n");
      return -1;
    }
  srand (time(NULL)); /* randomize the seed to ensure the list is
			 different in every execution */
  cmdCnt = 0;
  while ((cmdCnt < numCommand) & (year <= lastYear))
    {
      printActivity(year, month, date);
      if ((rand() % 3) == 0)
	{
	  date += rand() % 5;
	  if (date > dateMonth[month])
	    {
	      date = 1;
	      month ++;
	    }
	  if (month >= 13)
	    {
	      month = 1;
	      year ++;
	    }
	}
      cmdCnt ++;
    }
  return 0;
}
